"""
Auto-generated users resource.

DO NOT EDIT THIS FILE MANUALLY.
Run `python scripts/generate_resources.py` to regenerate.
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from ..client.base import BaseClient


class UsersResource:
    """
    User management.
    """

    def __init__(self, client: BaseClient) -> None:
        self._client = client

    def _build_params(self, **kwargs: Any) -> dict[str, Any]:
        """Build query parameters, filtering None values."""

        def to_camel(s: str) -> str:
            parts = s.split("_")
            return parts[0] + "".join(p.title() for p in parts[1:])

        return {to_camel(k): v for k, v in kwargs.items() if v is not None}

    def _build_payload(self, **kwargs: Any) -> dict[str, Any]:
        """Build request payload, filtering None values."""
        from datetime import datetime

        def to_camel(s: str) -> str:
            parts = s.split("_")
            return parts[0] + "".join(p.title() for p in parts[1:])

        result: dict[str, Any] = {}
        for k, v in kwargs.items():
            if v is None:
                continue
            if isinstance(v, datetime):
                result[to_camel(k)] = v.isoformat()
            else:
                result[to_camel(k)] = v
        return result

    def list_users(self) -> dict[str, Any]:
        """List team users (root + invited)"""
        return self._client._get("/v1/users")

    def get_user(self, user_id: str) -> dict[str, Any]:
        """Get user by id (self or invited)"""
        return self._client._get(f"/v1/users/{user_id}")

    async def alist_users(self) -> dict[str, Any]:
        """List team users (root + invited) (async)"""
        return await self._client._aget("/v1/users")

    async def aget_user(self, user_id: str) -> dict[str, Any]:
        """Get user by id (self or invited) (async)"""
        return await self._client._aget(f"/v1/users/{user_id}")
